package org.litesoft;

import junit.framework.*;

// Copyright Status:
//
// All Software available from LiteSoft.org (including this file) is
// hereby released into the public domain.
//
// It is free!  As in, you may use it freely in both commercial and
// non-commercial applications, bundle it with your software
// distribution, include it on a CD-ROM, list the source code in a book,
// mirror the documentation at your own web site, or use it in any other
// way you see fit.
//
// NO Warranty!
//
// All software is provided "as is".
//
// There is ABSOLUTELY NO WARRANTY OF ANY KIND: not for the design, fitness
// (for a particular purpose), level of errors (or lack thereof), or
// applicability of this software.  The entire risk as to the quality
// and performance of this software is with you.  Should this software
// prove defective, you assume the cost of all necessary servicing, repair
// or correction.
//
// In no event unless required by applicable law or agreed to in writing
// will any party who created or may modify and/or redistribute this
// software, be liable to you for damages, including any general,
// special, incidental or consequential damages arising out of the use or
// inability to use this software (including but not limited to loss of
// data or data being rendered inaccurate or losses sustained by you or
// third parties or a failure of this software to operate with any
// other programs), even if such holder or other party has been advised
// of the possibility of such damages.
//
// NOTE: Should you discover a bug, have a recogmendation for a change, wish
// to submit modifications, or wish to add new classes/functionality,
// please email them to:
//
//        changes@litesoft.org
//

public abstract class Extend4Test extends TestCase
{
    protected Extend4Test( String name )
    {
        super( name );
    }

    protected static void run( Test suite )
    {
        junit.textui.TestRunner.run( suite );
    }

    /**
     * Asserts that a condition is false. If it isn't it throws
     * an AssertionFailedError with the given message.
     */
    static public void assertFalse( String message , boolean condition )
    {
        if ( condition )
            fail( message );
    }

    /**
     * Asserts that a condition is false. If it isn't it throws
     * an AssertionFailedError.
     */
    static public void assertFalse( boolean condition )
    {
        assertFalse( null , condition );
    }

    /**
     * Asserts that two longs are NOT equal.
     * @param value1 the value1 value of an object
     * @param value2 the value2 value of an object
     */
    static public void assertNotEquals( long value1 , long value2 )
    {
        assertNotEquals( null , value1 , value2 );
    }

    /**
     * Asserts that two objects are NOT equal. If they are not
     * an AssertionFailedError is thrown.
     * @param value1 the value1 value of an object
     * @param value2 the value2 value of an object
     */
    static public void assertNotEquals( Object value1 , Object value2 )
    {
        assertNotEquals( null , value1 , value2 );
    }

    /**
     * Asserts that two longs are NOT equal.
     * @param message the detail message for this assertion
     * @param value1 the value1 value of an object
     * @param value2 the value2 value of an object
     */
    static public void assertNotEquals( String message , long value1 , long value2 )
    {
        assertNotEquals( message , new Long( value1 ) , new Long( value2 ) );
    }

    /**
     * Asserts that two objects are NOT equal. If they are not
     * an AssertionFailedError is thrown.
     * @param message the detail message for this assertion
     * @param value1 the value1 value of an object
     * @param value2 the value2 value of an object
     */
    static public void assertNotEquals( String message , Object value1 , Object value2 )
    {
        if ( value1 == value2 )
            failEquals( message , value1 );

        if ( value1 != null && value1.equals( value2 ) )
            failEquals( message , value1 );

        if ( value2 != null && value2.equals( value1 ) )
            failEquals( message , value2 );
    }

    static private void failEquals( String message , Object o )
    {
        String formatted = "";
        if ( message != null )
            formatted = "  " + message + "\n";
        throw new AssertionFailedError( formatted + "    value:<" + o + ">" );
    }
}